import json
import mimetypes
import os
import urllib
import uuid
import zipfile

from django.core.files.base import ContentFile
from django.core.files.storage import default_storage
import magic

from aio_client import configs as aio_client_settings
from kinder.core.declaration.models import DeclarationDoc


_PREF_EXT = {
    '.jpg',
    '.doc',
    '.docx',
    '.pdf',
    '.png',
    '.txt',
    '.odt',
    '.xls',
    '.xlsx',
}


def random_filename(file):
    """Возвращает случайное название файла."""
    # Пытаемся получить расширение из mime-типа
    mime_type = magic.from_buffer(file, mime=True)
    possible_extensions = mimetypes.guess_all_extensions(mime_type)
    extension = ''
    if possible_extensions:
        if len(possible_extensions) == 1:
            extension = possible_extensions[0]
        else:
            for _extension in possible_extensions:
                # Если несколько возможных расширений, отдаём предпочтение 
                # некоторым из них вместо того чтоб брать первое 
                if _extension in _PREF_EXT:
                    extension = _extension
                    break
            else:
                # Если предпочитаемых нет, берём первое
                extension = possible_extensions[0]
                    
    uid = uuid.uuid4().hex
    return f'{uid}{extension}'


def process_attachments(attachments, declaration, approve=True):
    """Обработка прикреплённых файлов."""

    docs = []

    # маневр на случай если в attachments прийдет JSON обернутый в список
    if isinstance(attachments, list):
        attachments = [json.loads(attachment) for attachment in attachments]
    else:
        attachments = json.loads(attachments)

    for attachment in attachments:

        url = attachment[0]
        url = urllib.parse.urljoin(aio_client_settings.AIO_SERVER, url)
        name, html = urllib.request.urlretrieve(url)

        try:
            with zipfile.ZipFile(name, "r",
                                 zipfile.ZIP_DEFLATED) as zip_archive:
                for zipped_file_name in zip_archive.namelist():
                    if zipped_file_name.split('.')[-1] == 'xml':
                        continue
                    file = zip_archive.read(zipped_file_name)
                    try:
                        filename = random_filename(file)
                    except Exception:
                        # Костыль, поэтому если не получилось - не расстраиваемся
                        filename = f'{uuid.uuid4().hex}.dat'

                    doc = DeclarationDoc.objects.create(
                        file=default_storage.save(
                            DeclarationDoc.DOWNLOAD_DIR +
                            os.path.basename(filename),
                            ContentFile(file)),
                        name=filename,
                        declaration=declaration,
                        approve=approve)
                    docs.append(doc)
        except zipfile.BadZipFile:
            pass

    return docs
